<?php
/**
 * @file
 * Contains the SunMailerContentBuilder class.
 */

/**
 * Builds the content for a newsletter.
 */
class SunMailerContentBuilder {
  /**
   * Newsletter to build the content for.
   *
   * @var SunMailerNewsletter
   */
  private $newsletter;

  /**
   * Newsletter content in the form of views and their results.
   *
   * @var array
   */
  private $newsletterViewsContent;

  /**
   * Caches the newsletter's content.
   *
   * @var array
   */
  private $newsletterContentCache;

  /**
   * Caches the raw XML for the newsletter's content.
   *
   * @var array
   */
  private $newsletterXmlCache;

  /**
   * DOMDocument for the current newsletter's content being built.
   */
  private $dom;

  /**
   * Initializes a new instance of the SunMailerContentBuilder class.
   *
   * @param $newsletter
   *   Newsletter to build the content from.
   */
  public function __construct($newsletter) {
    $this->newsletter = $newsletter;
    $sections = $this->newsletter->getSections();
    $this->newsletterViewsContent = array();
    foreach ($sections as $sid => $section) {
      $this->newsletterViewsContent[$sid] = SunMailerContentBuilder::loadSectionViewsContent($section, $newsletter->getCutoffDate());
    }
    $this->newsletterContentCache = array();
    $this->newsletterXmlCache = array();
  }

  /**
   * Gets the newsletter's content.
   *
   * @param $sids
   *   (Optional) IDs of the sections to include in the newsletter.
   * @param $useAlternateDesign
   *   (Optional) Whether to use the alernate design or the regular design.
   * @param $mail
   *   (Optional) Email address of the recipient
   *
   * @return
   *   Newsletter's content.
   */
  public function getNewsletterContent($sids = array(), $useAlternateDesign = FALSE, $mail = '') {
    // Check the cache.
    $sids = $this->cleanupSids($sids);
    $sidsKey = implode(',', $sids);
    $didKey = $useAlternateDesign ? $this->newsletter->getAlternateDesignId() : $this->newsletter->getDesignId();
    if (array_key_exists($sidsKey, $this->newsletterContentCache)) {
      if (array_key_exists($didKey, $this->newsletterContentCache[$sidsKey])) {
        $content = $this->newsletterContentCache[$sidsKey][$didKey];
        return $content ? SunMailerContentBuilder::populateSubscriptionLink($content, $mail) : NULL;
      }
    }

    // Build the newsletter XML and check that it exists.
    $xml = $this->buildNewsletterXml($sids);
    if (!$xml) {
      $this->newsletterContentCache[$sidsKey][$didKey] = NULL;
      return NULL;
    }

    // Load the XSL for the design.
    $design = $useAlternateDesign ? $this->newsletter->getAlternateDesign() : $this->newsletter->getDesign();
    if (!$design) {
      $this->newsletterContentCache[$sidsKey][$didKey] = NULL;
      return NULL;
    }

    $doc = new DOMDocument();
    $doc->load($design->getXsl()->filepath, LIBXML_NOCDATA);
    $xslt = new XSLTProcessor();
    $xslt->importStyleSheet($doc);

    // Create a newsletter with only absolute URLs.
    $content = $xslt->transformToXML($xml);
    global $base_url;
    $content = preg_replace('|href="/|', "href=\"$base_url/", $content);

    // Do additional processing for text newsletters.
    if (!$design->isHtml()) {
      $content = strip_tags($content);
      $content = html_entity_decode($content);
    }

    // Store the result in the cache.
    $this->newsletterContentCache[$sidsKey][$didKey] = $content;

    // Supply the subscription link.
    return SunMailerContentBuilder::populateSubscriptionLink($content, $mail);
  }

  /**
   * Gets the newsletter's content as XML.
   *
   * @param $sids
   *   (Optional) IDs of the sections to include in the newsletter.
   * @param $mail
   *   (Optional) Email address of the recipient
   *
   * @return
   *   Newsletter's content as XML.
   */
  public function getNewsletterXml($sids = array(), $mail = '') {
    $sids = $this->cleanupSids($sids);
    $xml = $this->buildNewsletterXml($sids);
    if (!$xml) {
      return NULL;
    }
    return SunMailerContentBuilder::populateSubscriptionLink($xml->saveXML(), $mail);
  }

  /**
   * Executes the view for a section to generate content.
   *
   * @param unknown_type $section
   *   Section to generate content for.
   * @param unknown_type $cutoffDate
   *   Cutoff date for results to exclude.
   *
   * @return
   *   Content from the results of the view.
   */
  private static function loadSectionViewsContent($section, $cutoffDate) {
    // Execute the display for the view.
    $view = views_get_view($section->getViewName());
    $view_args = count($section->getViewArguments()) ? explode('/', $section->getViewArguments()) : array();
    $view->execute_display($section->getViewDisplay(), $view_args);

    // Build the section content from the results.
    $sectionViewResults = array();
    foreach ($view->result as $rowKey => $row) {
      // Check if the node should be excluded.
      if (!$row->nid) {
        continue;
      }
      $node = node_load($row->nid);
      if ($node->created < $cutoffDate) {
        continue;
      }

      // Load the node and all fields provided by the view.
      $nodeFields = array();
      $nodeFields['node'] = $node;
      foreach ($view->field as $field => $value) {
        $nodeFields[$field] = $view->render_field($field, $rowKey);
      }
      $sectionViewResults[$row->nid] = $nodeFields;
    }
    return $sectionViewResults;
  }

  /**
   * Populates the subscription link within the newsletter's content.
   *
   * @param $content
   *   Newsletter's content.
   * @param $mail
   *   Email address of the recipient or nothing if the recipient is not known.
   *
   * @return
   *   Newsletter's content with the subscription link.
   */
  private static function populateSubscriptionLink($content, $mail) {
    $subscription_link = get_sunmailer_subscription_link($mail);
    return str_replace('!sunmailer-subscription-link', $subscription_link, $content);
  }

  /**
   * Cleans up the section IDs supplied by the user.
   *
   * @param $sids
   *   IDs of the sections to include, or nothing to include all sections.
   *
   * @return
   *   IDs of the section to include in the order they appear in the newsletter.
   */
  private function cleanupSids($sids) {
    $allSids = array_keys($this->newsletterViewsContent);
    $sids = count($sids) ? $sids : $allSids;
    return array_intersect($allSids, $sids);
  }

  /**
   * Builds the newsletter's content as XML.
   *
   * @param unknown_type $sids
   *   IDs of the sections to include.
   *
   * @return
   *   Newsletter's content as XML.
   */
  private function buildNewsletterXml($sids) {
    // Check the cache.
    $sidsKey = implode(',', $sids);
    if (array_key_exists($sidsKey, $this->newsletterXmlCache))
    {
      return $this->newsletterXmlCache[$sidsKey];
    }

    // Create the root newsletter node.
    $this->dom = new DOMDocument('1.0', variable_get('sunmailer_char_set', 'utf-8'));
    $newsletterNode = $this->dom->createElement('newsletter');
    $this->dom->appendChild($newsletterNode);

    // Build the newsletter XML.
    $sections = $this->newsletter->getSections();
    $emptyNewsletter = TRUE;

    $newsletterNode->appendChild($this->buildHeaderXml($this->newsletter));
    $sectionsNode = $this->dom->createElement('sections');
    foreach ($sids as $sid) {
      $section = $sections[$sid];
      $sectionViewsContent = $this->newsletterViewsContent[$sid];
      $sectionNode = $this->buildSectionXml($section, $sectionViewsContent);
      if ($sectionNode) {
        $emptyNewsletter = FALSE;
        $sectionsNode->appendChild($sectionNode);
      }
    }
    $newsletterNode->appendChild($sectionsNode);
    $newsletterNode->appendChild($this->buildFooterXml());

    // Check for an empty newsletter.
    if ($emptyNewsletter) {
      $newsletterNode = NULL;
    }

    // Allow developers to alter the XML.
    module_invoke_all('sunmailer_xml_newsletter_alter', $newsletterNode, $this->dom, $this->newsletter);
    if ($newsletterNode == NULL) {
      $this->dom = NULL;
    }

    // Store the XML in the cache.
    $this->newsletterXmlCache[$sidsKey] = $this->dom;
    return $this->dom;
  }

  /**
   * Builds the header for the newsletter's XML.
   *
   * @param $newsletter
   *   Newsletter to build the header for.
   *
   * @return
   *   Header for the newsletter's XML.
   */
  private function buildHeaderXml($newsletter) {
    // Retrieve the header information.
    $title = check_plain($newsletter->getTitle());
    if ($newsletter->getLogo()) {
      global $base_url;
      $logoPath = check_plain("$base_url/{$newsletter->getLogo()->filepath}");
    }
    $link = url('<front>', array('absolute' => TRUE));
    switch ($newsletter->getDateFormat()) {
      case '':
        $date = '';
        break;
      case 'small':
      case 'medium':
      case 'large':
        $date = format_date(time(), $newsletter->getDateFormat());
        break;
      default:
        $date = format_date(time(), 'custom', $newsletter->getDateFormat());
        break;
    }
    $message = filter_xss_admin($newsletter->getMessage());

    // Build the header XML.
    $headerNode = $this->dom->createElement('header');
    $headerNode->appendChild($this->buildLeafNode('title', $title));
    if ($logoPath) {
      $headerNode->appendChild($this->buildLeafNode('logo', $logoPath));
    }
    $headerNode->appendChild($this->buildLeafNode('link', $link));
    if ($date) {
      $headerNode->appendChild($this->buildLeafNode('date', $date));
    }
    if ($message) {
      $headerNode->appendChild($this->buildLeafNode('message', $message));
    }
    return $headerNode;
  }

  /**
   * Builds a section for the newsletter's XML.
   *
   * @param $section
   *   Section to build.
   * @param $sectionViewsContent
   *   Raw content for the section from the view.
   *
   * @return
   *   Section for the newsletter's XML.
   */
  private function buildSectionXml($section, $sectionViewsContent) {
    // Retrieve the section information.
    $title = (count($sectionViewsContent) > 1) ? check_plain($section->getTitle()) : '';

    // Build the section XML.
    $sectionNode = $this->dom->createElement('section');
    if ($title) {
      $sectionNode->appendChild($this->buildLeafNode('title', $title));
    }
    foreach ($sectionViewsContent as $viewsContent) {
      $sectionNode->appendChild($this->buildContentXml($viewsContent));
    }

    // Check for a section with no content.
    if (!count($sectionViewsContent)) {
      $sectionNode = NULL;
    }

    // Allow the developers to alter the XML.
    module_invoke_all('sunmailer_xml_section_alter', $sectionNode, $this->dom, $section, $sectionViewsContent);
    return $sectionNode;
  }

  /**
   * Builds a piece of content for the newsletter's XML.
   *
   * @param $viewsContent
   *   Raw content from the view.
   *
   * @return
   *   Piece of content for the newsletter's XML.
   */
  private function buildContentXml($viewsContent) {
    // Retrieve the content information.
    $node = $viewsContent['node'];
    $title = check_plain($node->title);
    $link = url("node/{$node->nid}", array('absolute' => TRUE));
    if ($node->uid) {
      $author = check_plain($node->name);
      $authorLink = url("user/{$node->uid}", array('absolute' => TRUE));
    }
    $date = format_date($node->created);
    $teaser = filter_xss_admin($node->teaser);

    // Build the basic content XML.
    $contentNode = $this->dom->createElement('content');
    $contentNode->appendChild($this->buildLeafNode('title', $title));
    $contentNode->appendChild($this->buildLeafNode('link', $link));
    if ($author) {
      $contentNode->appendChild($this->buildLeafNode('author', $author));
      $contentNode->appendChild($this->buildLeafNode('author-link', $authorLink));
    }
    $contentNode->appendChild($this->buildLeafNode('date', $date));
    $contentNode->appendChild($this->buildLeafNode('teaser', $teaser));

    // Add additional fields to the content XML.
    foreach ($viewsContent as $name => $field) {
      if ($name == 'node') {
        continue;
      }
      $contentNode->appendChild($this->buildLeafNode($name, $field));
    }

    // Allow developers to alter the XML.
    module_invoke_all('sunmailer_xml_content_alter', $this->dom, $viewsContent);
    return $contentNode;
  }

  /**
   * Builds the footer for the newsletter's XML.
   *
   * @return
   *   Footer for the newsletter's XML.
   */
  private function buildFooterXml() {
    $footerNode = $this->dom->createElement('footer');
    $footerNode->appendChild($this->buildLeafNode('subscription-link', '!sunmailer-subscription-link'));
    return $footerNode;
  }

  /**
   * Builds a leaf node.
   *
   * @param $name
   *   Name of the node.
   * @param $value
   *   Value of the node.
   *
   * @return
   *   Leaf node.
   */
  private function buildLeafNode($name, $value) {
    $leafNode = $this->dom->createElement($name);
    $leafNode->appendChild($this->dom->createCDATASection($value));
    return $leafNode;
  }
}
