<?php
/**
 * @file
 * Contains the SunMailerDesign class.
 */

/**
 * Represents a newsletter design.
 */
class SunMailerDesign {
  /**
   * Design id stored in the database.
   *
   * @var int
   */
  private $did;

  /**
   * Title stored in the database.
   *
   * @var string
   */
  private $title;

  /**
   * HTML or text flag stored in the database.
   *
   * @var bool
   */
  private $html;

  /**
   * File ID stored in the database for the XSL.
   *
   * @var int
   */
  private $xsl_fid;

  /**
   * XSL corresponding to the file ID stored in the database.
   *
   * @var object
   */
  private $xsl;

  /**
   * Original file ID loaded from the database for the XSL.
   */
  private $oldXslFid;

  /**
   * Creates a new design.
   *
   * @param $isHtml
   *   Whether the design is HTML or text.
   *
   * @return
   *   New design.
   */
  public static function createDesign($isHtml) {
    $design = new stdClass();
    $design->did = NULL;
    $design->title = '';
    $design->html = $isHtml;
    $design->fid = 0;
    return new SunMailerDesign($design);
  }

  /**
   * Loads a design from the database.
   *
   * @param $did
   *   Design's ID.
   *
   * @return
   *   Existing design or NULL if the design was not found.
   */
  public static function loadDesign($did) {
    $query = "SELECT * FROM {sunmailer_designs} WHERE did = %d";
    $designs = SunMailerDesign::loadDesigns($query, $did);
    return count($designs) ? $designs[$did] : NULL;
  }

  /**
   * Loads all designs from the database.
   *
   * @return
   *   All designs.
   */
  public static function loadAllDesigns() {
    $query = "SELECT * FROM {sunmailer_designs}";
    return SunMailerDesign::loadDesigns($query);
  }

  /**
   * Deletes a design from the database.
   *
   * @param $did
   *   ID of the design to delete.
   */
  public static function deleteDesign($did) {
    $design = SunMailerDesign::loadDesign($did);
    if (!$design) {
      return;
    }
    db_query("DELETE FROM {sunmailer_designs} WHERE did = %d", $design->getDesignId());
    db_query("DELETE FROM {files} WHERE fid = %d", $design->getXslFileId());
  }

  /**
   * Initializes a new instance of the SunMailerDesign class.
   *
   * @param $design
   *   Object containing the design's data.
   */
  private function __construct($design) {
    foreach ($design as $key => $value) {
      $this->$key = $design->$key;
    }
    $this->xsl = NULL;
    $this->oldXslFid = $this->xsl_fid;
  }

  /**
   * Gets the design's ID.
   *
   * @return
   *   Design's id.
   */
  public function getDesignId() {
    return $this->did;
  }

  /**
   * Gets the design's title.
   *
   * @return
   *   Design's title.
   */
  public function getTitle() {
    return $this->title;
  }

  /**
   * Sets the design's title.
   *
   * @param $title
   *   Design's title.
   */
  public function setTitle($title) {
    $this->title = $title;
  }

  /**
   * Gets whether the design is HTML or text.
   *
   * @return
   *   Whether the design is HTML or text.
   */
  public function isHtml() {
    return $this->html;
  }

  /**
   * Gets the file ID for the design's XSL.
   *
   * @return
   *   File ID for the design's XSL.
   */
  public function getXslFileId() {
    return $this->xsl_fid;
  }

  /**
   * Gets the design's XSL.
   *
   * @return
   *   Design's XSL.
   */
  public function getXsl() {
    if ($this->xsl == NULL){
      $this->xsl = db_fetch_object(db_query("SELECT * FROM {files} WHERE fid = %d", $this->xsl_fid));
    }
    return $this->xsl;
  }

  /**
   * Sets the file ID for the design's XSL.
   *
   * @param $xslFileId
   *   File ID for the design's XSL.
   */
  public function setXslFileId($xslFileId) {
    $this->xsl = NULL;
    $this->xsl_fid = $xslFileId;
  }

  /**
   * Saves the design in the database.
   */
  public function save() {
    // Save the design.
    file_set_status($this->getXsl(), FILE_STATUS_PERMANENT);

    $design = new stdClass();
    foreach ($this as $key => $value) {
      $design->$key = $this->$key;
    }
    drupal_write_record('sunmailer_designs', $design, $design->did ? 'did' : array());

    // Delete old files.
    if ($this->oldXslFid != $this->xsl_fid) {
      db_query("DELETE FROM {files} WHERE fid = %d", $this->oldXslFid);
      $this->oldXslFid = $this->xsl_fid;
    }
  }

  /**
   * Loads a collection of designs from the database.
   *
   * @param $query
   *   Query to retrieve the designs.
   * @param $args
   *   (optional) Query arguments.
   */
  private static function loadDesigns($query, $args = array()) {
    $designs = array();
    $result = db_query($query, $args);
    while ($row = db_fetch_object($result)) {
      $designs[$row->did] = new SunMailerDesign($row);
    }
    return $designs;
  }
}
