<?php
/**
 * @file
 * Contains the SunMailerSchedule class.
 */

/**
 * Represents a newsletter mailing schedule.
 */
class SunMailerSchedule {
  /**
   * ID of the newsletter this schedule is for.
   *
   * @var int
   */
  private $nlid;

  /**
   * Type of mailing schedule.
   *
   * @var string
   */
  private $type;

  /**
   * Time unit for interval and absolute schedules.
   *
   * @var string
   */
  private $unit;

  /**
   * Next date to mail the newsletter.
   *
   * @var int
   */
  private $nextDate;

  /**
   * Time unit for incrementing the date to the next scheduled mailing.
   *
   * @var string
   */
  private $deltaUnit;

  /**
   * Interval between mailings for an interval schedule.
   *
   * @var int
   */
  private $interval;

  /**
   * PHP date format for selecting the date for an absolute schedule.
   *
   * @var string
   */
  private $selectionFormat;

  /**
   * Date selection for an absolute schedule.
   *
   * @var array
   */
  private $selection;

  /**
   * Initializes a new instance of the SunMailerSchedule class.
   *
   * @param $schedule
   *   Object containing the schedule data.
   */
  private function __construct($schedule) {
    // Set the common schedule information.
    $this->nlid = $schedule->nlid;
    $this->type = $schedule->type;
    $this->nextDate = $schedule->next_date;

    // Set the schedule information dependent on schedule type.
    switch ($this->type) {
    case 'interval':
      $this->setIntervalUnit($schedule->unit);
      $this->interval = $schedule->data;
      break;
    case 'absolute':
      $this->setAbsoluteUnit($schedule->unit);
      $this->selection = explode(',', $schedule->data);
      break;
    case 'single':
    case 'none':
      return;
    }
  }

  /**
   * Loads a newsletter's schedule from the database.
   *
   * @param $nlid
   *   Newsletter's ID.
   *
   * @return
   *   New or existing schedule.
   */
  public static function loadSchedule($nlid) {
    // First try loading the schedule from the database.
    $query = "SELECT * FROM sunmailer_schedules WHERE nlid = %d";
    $schedule = db_fetch_object(db_query($query, $nlid));
    if (!$schedule) {
      // Create a new schedule.
      $schedule = new stdClass();
      $schedule->nlid = $nlid;
      $schedule->type = 'none';
      $schedule->unit = '';
      $schedule->data = '';
      $schedule->next_date = 0;
    }
    return new SunMailerSchedule($schedule);
  }

  /**
   * Gets the ID of the newsleter this schedule is for.
   *
   * @return
   *   Newsletter's ID.
   */
  public function getNewsletterId() {
    return $this->nlid;
  }

  /**
   * Gets the schedule's type.
   *
   * @return
   *   Schedule's type.
   */
  public function getType() {
    return $this->type;
  }

  /**
   * Gets the time unit for an interval or absolute schedule.
   *
   * @return
   *   Time unit for the schedule.
   */
  public function getUnit() {
    switch ($this->type) {
    case 'interval':
    case 'absolute':
      break;
    default:
      return NULL;
    }
    return $this->unit;
  }

  /**
   * Gets the interval for an interval schedule.
   *
   * @return
   *   Interval between mailings
   */
  public function getInterval() {
    if ($this->type != 'interval') {
      return NULL;
    }
    return $this->interval;
  }

  /**
   * Gets the selection of times for an absolute schedule.
   *
   * @return
   *   Selection of times to send the mailing.
   */
  public function getSelection() {
    if ($this->type != 'absolute') {
      return NULL;
    }
    return $this->selection;
  }

  /**
   * Gets the date of the next schedule mailing.
   *
   * @return
   *   Date of the next scheduled mailing.
   */
  public function getNextDate() {
    if ($this->type == 'none') {
      return NULL;
    }
    return $this->nextDate;
  }

  /**
   * Schedules a single mailing.
   *
   * @param $nextDate
   *   Date of the mailing.
   */
  public function scheduleOnce($nextDate) {
    // Validate the schedule.
    if (!is_numeric($nextDate)) {
      return FALSE;
    }

    // Set the schedule.
    $this->type = 'single';
    $this->nextDate = $nextDate;
    return TRUE;
  }

  /**
   * Schdules a mailing at a given interval (e.g.: every 2 days).
   *
   * @param $unit
   *   Interval unit (e.g.: 'day')
   * @param $interval
   *   Interval (e.g.: 2)
   * @param $nextDate
   *   Earliest date to start the mailings.
   */
  public function scheduleAtInterval($unit, $interval, $nextDate) {
    // Validate the schedule.
    switch ($unit) {
    case 'minute':
    case 'hour':
    case 'day':
    case 'week':
    case 'month':
      break;
    default:
      return FALSE;
    }
    if (!is_numeric($interval) || ($interval <= 0)) {
      return FALSE;
    }
    if (!is_numeric($nextDate)) {
      return FALSE;
    }

    // Set the schedule.
    $this->type = 'interval';
    $this->setIntervalUnit($unit);
    $this->interval = $interval;
    $this->nextDate = $nextDate;
    return TRUE;
  }

  /**
   * Schedules a mailing at a specific time (e.g.: 1st and 16th day of each month).
   *
   * @param $unit
   *   Schedule unit (e.g.: 'month').
   * @param $selection
   *   Date or time selection (e.g.: array(1, 16)).
   * @param $nextDate
   *   Earliest date to start the mailings.
   */
  public function scheduleAtSpecifiedDates($unit, $selection, $nextDate) {
    // Validate the schedule.
    switch ($unit) {
    case 'hour':
      $validSelection = $this->validateSelection($selection, 0, 59);
      break;
    case 'day':
      $validSelection = $this->validateSelection($selection, 0, 23);
      break;
    case 'week':
      $validSelection = $this->validateSelection($selection, 0, 6);
      break;
    case 'month':
      $validSelection = $this->validateSelection($selection, 1, 31);
      break;
    default:
      return FALSE;
    }
    if (!$validSelection) {
      return FALSE;
    }
    if (!is_numeric($nextDate)) {
      return FALSE;
    }

    // Set the schedule.
    $this->type = 'absolute';
    $this->setAbsoluteUnit($unit);
    $this->selection = array_unique($selection, SORT_NUMERIC);
    sort($this->selection, SORT_NUMERIC);
    $this->nextDate = $nextDate;
    $this->updateNextDate(0);
    return TRUE;
  }

  /**
   * Deletes the current mailing schedule.
   */
  public function deleteSchedule() {
    $this->type = 'none';
  }

  /**
   * Checks if a mailing can occur and updates the next mailing date if so.
   *
   * @return
   *   True if a mailing can occur.
   */
  public function checkSchedule() {
    // Check if a mailing can occur.
    if ($this->type == 'none') {
      return FALSE;
    }
    $now = time();
    if ($this->nextDate > $now) {
      return FALSE;
    }

    // Update the next mailing date.
    switch ($this->type) {
    case 'single':
      $this->type = 'none';
      break;
    case 'interval':
    case 'absolute':
      $this->updateNextDate($now);
      break;
    }
    return TRUE;
  }

  /**
   * Saves the schedule in the database.
   */
  public function save()
  {
    // Don't store deleted schedules.
    if ($this->type == 'none') {
      $query = "DELETE FROM {sunmailer_schedules} WHERE nlid = %d";
      db_query($query, $this->nlid);
      return;
    }

    // Determine if a schedule already exists in the database.
    $query = "SELECT COUNT(*) FROM {sunmailer_schedules} WHERE nlid = %d";
    $update = db_result(db_query($query, $this->nlid));

    // Save the schedule.
    $schedule = new stdClass();
    $schedule->nlid = $this->nlid;
    $schedule->type = $this->type;
    $schedule->unit = $this->unit;
    $schedule->next_date = $this->nextDate;
    switch ($this->type) {
    case 'interval':
      $schedule->data = $this->interval;
      break;
    case 'absolute':
      $schedule->data = implode(',', $this->selection);
      break;
    default:
      $schedule->data = '';
      break;
    }
    drupal_write_record('sunmailer_schedules', $schedule, $update ? 'nlid' : array());
  }

  /**
   * Checks if the selection is one or more numbers within a certain range.
   *
   * @param $selection
   *   Array of numbers.
   * @param $low
   *   Low value of the range, inclusive.
   * @param $high
   *   High value of the range, inclusive.
   *
   * @return
   *   True if at least one number exists and the numbers are within the range.
   */
  private static function validateSelection($selection, $low, $high)
  {
    if (!is_array($selection) || (count($selection) == 0)) {
      return FALSE;
    }
    foreach ($selection as $value) {
      if (!is_numeric($value) || ($value < $low) || ($value > $high)) {
        return FALSE;
      }
    }
    return TRUE;
  }

  /**
   * Sets the unit and other related fields for interval schedules.
   *
   * @param unit
   *   Time unit for the schedule
   */
  private function setIntervalUnit($unit) {
    $this->unit = $unit;
    $this->deltaUnit = $unit;
  }

  /**
   * Sets the unit an other related fields for absolute schedules.
   *
   * @param unit
   *   Time unit for the schedule
   */
  private function setAbsoluteUnit($unit)
  {
    $this->unit = $unit;
    switch ($this->unit) {
    case 'hour':
      $this->deltaUnit = 'minute';
      $this->selectionFormat = 'i';
      break;
    case 'day':
      $this->deltaUnit = 'hour';
      $this->selectionFormat = 'G';
      break;
    case 'week':
      $this->deltaUnit = 'day';
      $this->selectionFormat = 'w';
      break;
    case 'month':
      $this->deltaUnit = 'day';
      $this->selectionFormat = 'j';
      break;
    }
  }

  /**
   * Increments the next date to the first scheduled date after the current date.
   */
  private function updateNextDate($currentDate) {
    $datetime = new DateTime(date('Y-m-d H:i', $this->nextDate));
    switch ($this->type) {
    case 'interval':
      // Increment by the interval until we pass the current date.
      $increment = "+$this->interval $this->deltaUnit";
      while ($datetime->format('U') <= $currentDate) {
        $datetime->modify($increment);
      }
      $this->nextDate = $datetime->format('U');
      break;

    case 'absolute':
      // Increment until we pass the current date.
      $increment = "+1 $this->deltaUnit";
      while ($datetime->format('U') <= $currentDate) {
        $datetime->modify($increment);
      }

      // Increment until we reach one of the selected dates.
      while (!in_array($datetime->format($this->selectionFormat), $this->selection))
      {
        $datetime->modify($increment);
      }
      $this->nextDate = $datetime->format('U');
      break;
    }
  }
}
