<?php
/**
 * @file
 * Contains the SunMailerSection class.
 */

/**
 * Represents a section of content in a newsletter.
 */
class SunMailerSection {
  /**
   * Section ID stored in the database.
   *
   * @var int
   */
  private $sid;

  /**
   * Newsletter ID stored in the database.
   *
   * @var int
   */
  private $nlid;

  /**
   * Title stored in the database.
   *
   * @var string
   */
  private $title;

  /**
   * View name stored in the database.
   *
   * @var string
   */
  private $view;

  /**
   * View display stored in the database.
   *
   * @var string
   */
  private $display;

  /**
   * View arguments, separeted by '/', stored in the database.
   *
   * @var string
   */
  private $args;

  /**
   * Weight stored in the database.
   *
   * @var int
   */
  private $weight;

  /**
   * Initializes a new instance of the SunMailerSection class.
   *
   * @param $section
   *   Object containing the section's data.
   */
  private function __construct($section) {
    foreach ($section as $key => $value) {
      $this->$key = $section->$key;
    }
  }

  /**
   * Creates a new section.
   *
   * @param $nlid
   *   Id of the newsletter to add this section to.
   * @param $weight
   *   (optional) Section's weight.
   *
   * @return
   *   New section.
   */
  public static function createSection($nlid, $weight = 0) {
    $section = new stdClass();
    $section->sid = NULL;
    $section->nlid = $nlid;
    $section->title = '';
    $section->view = '';
    $section->display = '';
    $section->args = '';
    $section->weight = $weight;
    return new SunMailerSection($section);
  }

  /**
   * Loads a section from the database.
   *
   * @param $sid
   *   Section's ID.
   *
   * @return
   *   Existing section or NULL if the section was not found.
   */
  public static function loadSection($sid) {
    $query = "SELECT * FROM {sunmailer_sections} WHERE sid = %d ORDER BY weight, sid ASC";
    $sections = SunMailerSection::loadSections($query, $sid);
    return count($sections) ? $sections[$sid] : NULL;
  }

  /**
   * Loads all sections from the database in weighted order.
   *
   * @return
   *   All sections.
   */
  public static function loadAllSections() {
    $query = "SELECT * FROM {sunmailer_sections} ORDER BY weight, sid ASC";
    return SunMailerSection::loadSections($query);
  }

  /**
   * Loads all sections of a newsletter from the database in weighted order.
   *
   * @param $nlid
   *   Newsletter's ID.
   *
   * @return
   *   All sections of the newsletter.
   */
  public static function loadNewsletterSections($nlid) {
    $query = "SELECT * FROM {sunmailer_sections} WHERE nlid = %d ORDER BY weight, sid ASC";
    return SunMailerSection::loadSections($query, $nlid);
  }

  /**
   * Deletes a section from the database.
   *
   * @param $sid
   *   ID of the section to delete.
   */
  public static function deleteSection($sid) {
    db_query("DELETE FROM {sunmailer_sections} WHERE sid = %d", $sid);
  }

  /**
   * Gets the section's ID.
   *
   * @return
   *   Section's ID.
   */
  public function getSectionId() {
    return $this->sid;
  }

  /**
   * Gets the ID of the newsletter the section belongs to.
   *
   * @return
   *   Newsletter's ID.
   */
  public function getNewsletterId() {
    return $this->nlid;
  }

  /**
   * Gets the section's title.
   *
   * @return
   *   Section's title.
   */
  public function getTitle() {
    return $this->title;
  }

  /**
   * Sets the section's title.
   *
   * @param $title
   *   Section's title.
   */
  public function setTitle($title) {
    $this->title = $title;
  }

  /**
   * Gets the name of the section's view.
   *
   * @return
   *   View's name.
   */
  public function getViewName() {
    return $this->view;
  }

  /**
   * Gets the display for the section's view.
   *
   * @return
   *   View's display.
   */
  public function getViewDisplay() {
    return $this->display;
  }

  /**
   * Gets the arguments for the section's view.
   *
   * @return
   *   View's arguments, separated by '/'.
   */
  public function getViewArguments() {
    return $this->args;
  }

  /**
   * Sets the view.
   *
   * @param $view
   *   View's name.
   * @param $display
   *   View's display.
   * @param $args
   *   (optional) View's arguments, separated by '/'.
   */
  public function setView($view, $display, $args = '') {
    $this->view = $view;
    $this->display = $display;
    $this->args = $args;
  }

  /**
   * Gets the section's weight.
   *
   * @return
   *   Section's weight.
   */
  public function getWeight() {
    return $this->weight;
  }

  /**
   * Sets the section's weight.
   *
   * @param $weight
   *   Section's weight.
   */
  public function setWeight($weight) {
    $this->weight = weight;
  }

  /**
   * Saves the section in the database.
   */
  public function save()
  {
    $section = new stdClass();
    foreach ($this as $key => $value) {
      $section->$key = $this->$key;
    }
    drupal_write_record('sunmailer_sections', $section, $section->sid ? 'sid' : array());
    $this->sid = $section->sid;
  }

  /**
   * Loads a collection of sections from the database.
   *
   * @param $query
   *   Query to retrieve the sections.
   * @param $args
   *   Query arguments.
   *
   * @return
   *   All sections retrieved by the query.
   */
  private static function loadSections($query, $args = array()){
    $sections = array();
    $result = db_query($query, $args);
    while ($row = db_fetch_object($result)) {
      $sections[$row->sid] = new SunmailerSection($row);
    }
    return $sections;
  }
}
