<?php
/**
 * @file
 * Contains the SunMailerStatistics class.
 */

/**
 * Calculates subscription statistics for all the newsletters.
 */
class SunMailerStatistics {
  /**
   * Subscription statistics for all newsletters.
   *
   * @var array
   */
  private $statistics;

  /**
   * Initializes a new instance of the SunMailerStatistics class.
   */
  public function __construct() {
    $this->statistics = SunMailerStatistics::calculateAllStatistics();
  }

  /**
   * Gets the subscription statistics for all newsletters.
   *
   * @return
   *   Subscription statistics for all newsletters.
   */
  public function getAllStatistics() {
    return $this->statistics;
  }

  /**
   * Gets the subscription statistics a single newsletter.
   *
   * @param $nlid
   *   Newsletter's ID.
   *
   * @return
   *   Subscription statistics for the newsletter.
   */
  public function getNewsletterStatistics($nlid) {
    if (!isset($this->statistics['newsletters'][$nlid])) {
      return NULL;
    }
    return $this->statistics['newsletters'][$nlid];
  }

  /**
   * Calculates the subscription statistics for all newsletters.
   *
   * @return
   *   Subscription statistics for all newsletters.
   */
  private static function calculateAllStatistics() {
    $newsletters = SunMailerNewsletter::loadAllNewsletters();
    $subscription_aggregator = get_sunmailer_subscription_aggregator();

    // Initialize the statistics.
    $statistics = array(
      'count' => 0,
      'newsletters' => array(),
    );

    // Process the subscriptions.
    $statistics['count'] = count($subscription_aggregator->getAllSubscribers());
    foreach ($newsletters as $nlid => $newsletter) {
      $newsletter_subscriptions = $subscription_aggregator->getNewsletterSubscriptions($nlid);
      $statistics['newsletters'][$nlid] = SunMailerStatistics::calculateNewsletterStatistics($newsletter, $newsletter_subscriptions);
    }
    return $statistics;
  }

  /**
   * Calculates the subscription statistics for a single newsletter.
   *
   * @param $newsletter
   *   Newsletter to calculate subscription statistics for.
   * @param $subscriptions
   *   Subscriptions to the newsletter.
   *
   * @return
   *   Subscription statistics for the newsletter.
   */
  private static function calculateNewsletterStatistics($newsletter, $subscriptions) {
    $sections = $newsletter->getSections();

    // Initialize the statistics.
    $statistics = array(
      'title' => $newsletter->getTitle(),
      'count' => 0,
      'sections' => array(),
    );
    foreach ($sections as $sid => $section) {
      $statistics['sections'][$sid] = array(
        'title' => $section->getTitle(),
        'count' => 0,
      );
    }

    // Process the subscriptions.
    foreach ($subscriptions as $mail => $sids) {
      $statistics['count']++;
      foreach ($sids as $sid) {
        $statistics['sections'][$sid]['count']++;
      }
    }
    return $statistics;
  }
}