<?php

/**
 * @file
 * Schedule the newsletter or preview and send it manually.
 */

/**
 * Checks the newsletter for errors.
 */
function sunmailer_check_newsletter() {
  module_load_install('sunmailer');
  $errors = array();

  // Check the module's requirements.
  $requirements = module_invoke('sunmailer', 'requirements', 'runtime');
  if (count($requirements)) {
    foreach ($requirements as $requirement) {
      $errors[] = $requirement['description'];
    }
    return $errors;
  }

  // Check the sections' views.
  $sections = sunmailer_get_sections();
  views_get_all_views(); // http://drupal.org/node/670308
  foreach ($sections as $section) {
    $name = $section['view'];
    $display = $section['display'];
    $view = views_get_view($name);
    if (!$view) {
      $errors[] = t('View !name does not exist.', array('!name' => $name));
      continue;
    }
    if ($view->base_table != 'node') {
      $errors[] = t('View !name is not a node view.', array('!name' => $name));
      continue;
    }
    if ($view->disabled) {
      $errors[] = t('View !name is disabled.', array('!name' => $name));
      continue;
    }
    if (!array_key_exists($display, $view->display)) {
      $errors[] = t('View !name does not contain the display !display.', array('!name' => $name, '!display' => $display));
      continue;
    }
  }
  if (count($errors)) {
    return $errors;
  }

  // Check the design's XSL file.
  $xml = sunmailer_xml_newsletter(array_keys(sunmailer_get_sections()));
  if (!$xml) {
    return array();
  }
  $formats = sunmailer_get_formats();
  foreach ($formats as $format) {
    $row = db_fetch_object(db_query("SELECT filename, filepath FROM {files} WHERE fid = %d", $format['fid']));
    $xsl_doc = new DOMDocument();
    if (!$xsl_doc->load($row->filepath)) {
      $errors[] = t('!name is not a valid XSL file.', array('!name' => $row->filename));
      continue;
    }
    $xslt = new XSLTProcessor();
    $xslt->importStyleSheet($xsl_doc);
    if (!$xslt->transformToXml($xml)) {
      $errors[] = t('An XSLT error occurred when transforming the XML with !name.', array('!name' => $row->filename));
    }
  }
  return $errors;
}

function sunmailer_newsletter_preview($newsletter, $preview_type) {
  $content_builder = new SunMailerContentBuilder($newsletter);
  $xml = $content_builder->getNewsletterXml();
  if (!$xml) {
    print 'The newsletter currently has no content';
    exit;
  }

  switch ($preview_type) {
  case 'design':
    $html = $content_builder->getNewsletterContent();
    print $html;
    exit;

  case 'alternate_design':
    $text = $content_builder->getNewsletterContent(array(), TRUE);
    $html = nl2br(htmlentities($text));
    print $html;
    exit;

  case 'xml':
    drupal_set_header('Content-Type: text/xml');
    print $xml;
    exit;
  }
}

/**
 * Mails the newsletter.
 */
function sunmailer_send_newsletter($newsletter) {
  // Check for errors.
  /*$errors = sunmailer_check_newsletter();
  if (count($errors)) {
    watchdog('mail', 'Error(s) sending the newsletter: ' . implode(' ', $errors), array(), WATCHDOG_ERROR);
    return;
  }*/

  // Check if the newsletter has any content.
  $content_builder = new SunMailerContentBuilder($newsletter);
  $xml = $content_builder->getNewsletterXml();
  if (!$xml) {
    watchdog('mail', 'Skipping the newsletter; it has no new content.');
    return;
  }

  // Log the beginning of the mailing.
  watchdog('mail', 'Sending the newsletter');
  $mailer = new SunMailerMailer();
  $start_time = time();

  // Retrieve the subscriptions grouped by sections subscribed to.
  $subscription_aggregator = get_sunmailer_subscription_aggregator();
  $grouped_subscriptions = $subscription_aggregator->getGroupedNewsletterSubscriptions($newsletter->getNewsletterId());

  // Send the appropriate newsletter to each subscription group.
  foreach ($grouped_subscriptions as $sid_list => $addresses) {
    $sids = explode(',', $sid_list);
    if (variable_get('sunmailer_max_recipients', 1) == 1) {
      // Each subscriber's newsletter is slightly different due to the custom subscription link.
      foreach ($addresses as $address) {
        $body = $content_builder->getNewsletterContent($sids, FALSE, $address);
        $alt_body = $content_builder->getNewsletterContent($sids, TRUE, $address);
        $mailer->sendMail(array($address), $newsletter->getSubject(), $body, $newsletter->getDesign()->isHtml(), $alt_body);
      }
    }
    else {
      // All subscribers get the same newsletter.
      $body = $content_builder->getNewsletterContent($sids, FALSE);
      $alt_body = $content_builder->getNewsletterContent($sids, TRUE);
      $mailer->sendMail($addresses, $newsletter->getSubject(), $body, $newsletter->getDesign()->isHtml(), $alt_body);
    }
  }

  // Log the end of the mailing.
  watchdog('mail', 'Sent the newsletter');
  if ($newsletter->hasCutoffDate()) {
    $newsletter->setCutoffDate($start_time);
  }
}

/**
 * Generates a form for newsletter settings.
 */
function sunmailer_mail_settings($form_state = NULL, $nlid) {
  $newsletter = SunMailerNewsletter::loadNewsletter($nlid);
  $form = array();
  $form['nlid'] = array(
    '#type' => 'value',
    '#value' => $nlid,
  );
  $form['cutoff'] = array(
    '#type' => 'fieldset',
    '#title' => 'Cutoff date',
    '#collapsible' => FALSE,
  );
  $form['cutoff']['has_cutoff_date'] = array(
    '#type' => 'checkbox',
    '#title' => t('Exclude content created before a specific date'),
    '#default_value' => $newsletter->hasCutoffDate(),
  );
  $form['cutoff']['cutoff_date'] = array(
    '#title' => t('Cutoff date'),
    '#required' => TRUE,
    '#default_value' => format_date($newsletter->hasCutoffDate() ? $newsletter->getCutoffDate() : time(), 'custom', 'Y-m-d H:i'),
  );
  _sunmailer_set_date_type($form['cutoff']['cutoff_date']);
  $form['message'] = array(
    '#type' => 'textarea',
    '#title' => t('Message'),
    '#description' => t('(Optional) This message will be displayed in the header of the newsletter'),
    '#default_value' => $newsletter->hasMessage() ? $newsletter->getMessage() : '',
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
  );
  return $form;
}

/**
 * Submission callback for newsletter settings form.
 */
function sunmailer_mail_settings_submit($form, &$form_state) {
  $values = $form_state['values'];
  $newsletter = SunMailerNewsletter::loadNewsletter($values['nlid']);
  if ($values['has_cutoff_date']) {
    $newsletter->setCutoffDate(strtotime($values['cutoff_date']));
  }
  else {
    $newsletter->deleteCutoffDate();
  }
  if ($values['message']) {
    $newsletter->setMessage($values['message']);
  }
  else {
    $newsletter->deleteMessage();
  }
  $newsletter->save();
}

/**
 * Generates a form to schedule the newsletter.
 */
function sunmailer_mail_schedule($form_state = NULL, $nlid) {
  $schedule = SunMailerSchedule::loadSchedule($nlid);

  $form = array();
  $form['nlid'] = array(
    '#type' => 'value',
    '#value' => $nlid,
  );

  // Build all the options for scheduling.
  $form['schedule']['type'] = array(
    '#type' => 'radios',
    '#options' => array(
      'none' => 'No schedule',
      'single' => 'one time only',
      'minute_interval' => 'every !field minute(s)',
      'hour_interval' => 'every !field hour(s)',
      'day_interval' => 'every !field days(s)',
      'week_interval' => 'every !field week(s)',
      'month_interval' => 'every !field month(s)',
      'hour_absolute' => 'on minute(s) !field of every hour (0-59)',
      'day_absolute' => 'on hour(s) !field of every day (0-23)',
      'week_absolute' => 'on the following day(s) of every week:',
      'month_absolute' => 'on day(s) !field of every month (1-31)',
    ),
  );

  // Build the settings for each option.
  $types = array('minute_interval', 'hour_interval', 'day_interval', 'week_interval', 'month_interval');
  foreach ($types as $type) {
    $form['schedule'][$type] = array(
      '#type' => 'textfield',
      '#size' => 2,
    );
  }
  $types = array('hour_absolute', 'day_absolute', 'month_absolute');
  foreach ($types as $type) {
    $form['schedule'][$type] = array(
      '#type' => 'textfield',
      '#size' => 8,
    );
  }
  $form['schedule']['week_absolute'] = array(
    '#type' => 'checkboxes',
    '#options' => array(
      0 => t('Sunday'),
      1 => t('Monday'),
      2 => t('Tuesday'),
      3 => t('Wednesday'),
      4 => t('Thursday'),
      5 => t('Friday'),
      6 => t('Saturday'),
    ),
  );

  // Set the defaults.
  switch ($schedule->getType()) {
    case 'none':
    case 'single':
      $form['schedule']['type']['#default_value'] = $schedule->getType();
      break;

    case 'interval':
      $default_type = "{$schedule->getUnit()}_interval";
      $form['schedule']['type']['#default_value'] = $default_type;
      $form['schedule'][$default_type]['#default_value'] = $schedule->getInterval();
      break;

    case 'absolute':
      $default_type = "{$schedule->getUnit()}_absolute";
      $default_selection = $schedule->getSelection();
      if ($schedule->getUnit() != 'week') {
        $default_selection = implode(', ', $default_selection);
      }
      $form['schedule']['type']['#default_value'] = $default_type;
      $form['schedule'][$default_type]['#default_value'] = $default_selection;
      break;
  }

  // Build the field to set the next date.
  $form['schedule']['next_date'] = array(
    '#title' => t('Send the next newsletter no sooner than'),
    '#default_value' => format_date(($schedule->getType() != 'none') ? $schedule->getNextDate() : time(), 'custom', 'Y-m-d H:i'),
  );
  _sunmailer_set_date_type($form['schedule']['next_date']);

  // Build the rest of the form.
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
  );

  return $form;
}

function sunmailer_mail_schedule_submit($form, &$form_state) {
  $values = $form_state['values'];
  $schedule = SunMailerSchedule::loadSchedule($values['nlid']);
  $type = $values['type'];
  $next_date = strtotime($values['next_date']);

  // Process no schedule.
  if ($type == 'none') {
    $schedule->deleteSchedule();
  }

  // Process a single schedule.
  if ($type == 'single') {
    $schedule->scheduleOnce($next_date);
  }

  // Process an interval schedule.
  if (strpos($type, '_interval') !== FALSE) {
    $type_parts = explode('_', $type);
    $unit = $type_parts[0];
    $interval = $values[$type];
    $schedule->scheduleAtInterval($unit, $interval, $next_date);
  }

  // Process an absolute schedule.
  if (strpos($type, '_absolute') !== FALSE) {
    $type_parts = explode('_', $type);
    $unit = $type_parts[0];
    if ($unit == 'week') {
      $selection = array_filter($values[$type]);
    }
    else {
      $selection = preg_split('/\\D+/', $values[$type], NULL, PREG_SPLIT_NO_EMPTY);
    }
    $schedule->scheduleAtSpecifiedDates($unit, $selection, $next_date);
  }

  // Save the schedule.
  $schedule->save();
}

function theme_sunmailer_mail_schedule($form) {
  $schedule = $form['schedule'];
  $type = $schedule['type'];
  drupal_add_css(drupal_get_path('module', 'sunmailer') . '/sunmailer.mail-schedule.css');
  $html = '';

  // Do not schedule the newsletter.
  $fieldset = array(
    '#title' => t('Do not schedule the newsletter to be sent'),
    '#collapsible' => FALSE,
    '#value' => _sunmailer_render_schedule_option($type['none']),
    '#attributes' => array(),
  );
  $html .= theme('fieldset', $fieldset);

  // Schedule the newsletter.
  $value = '';
  $value .= _sunmailer_render_schedule_option($type['single']);
  $value .= _sunmailer_render_schedule_option($type['minute_interval'], $schedule['minute_interval']);
  $value .= _sunmailer_render_schedule_option($type['hour_interval'], $schedule['hour_interval']);
  $value .= _sunmailer_render_schedule_option($type['day_interval'], $schedule['day_interval']);
  $value .= _sunmailer_render_schedule_option($type['week_interval'], $schedule['week_interval']);
  $value .= _sunmailer_render_schedule_option($type['month_interval'], $schedule['month_interval']);
  $value .= _sunmailer_render_schedule_option($type['hour_absolute'], $schedule['hour_absolute']);
  $value .= _sunmailer_render_schedule_option($type['day_absolute'], $schedule['day_absolute']);
  $value .= _sunmailer_render_schedule_option($type['week_absolute']);
  $value .= drupal_render($schedule['week_absolute']);
  $value .= _sunmailer_render_schedule_option($type['month_absolute'], $schedule['month_absolute']);
  $value .= drupal_render($schedule['next_date']);

  $fieldset = array(
    '#title' => t('Schedule the newsletter to be sent...'),
    '#collapisble' => FALSE,
    '#value' => $value,
    '#attributes' => array(),
  );
  $html .= theme('fieldset', $fieldset);

  // Render the rest of the form.
  unset($form['schedule']);
  $html .= drupal_render($form);
  return $html;
}

function _sunmailer_render_schedule_option($option, $field = NULL) {
  $option['#title'] = t($option['#title'], array('!field' => drupal_render($field)));
  return drupal_render($option);
}

/**
 * Generates a form to send the newsletter.
 */
function sunmailer_mail_send($form_state = NULL, $nlid) {
  $newsletter = SunMailerNewsletter::loadNewsletter($nlid);

  $form = array();
  $form['nlid'] = array(
    '#type' => 'value',
    '#value' => $newsletter->getNewsletterId(),
  );

  // Check for errors.
  /*$errors = sunmailer_check_newsletter();
  if (count($errors)) {
    drupal_set_title('Error');
    $html = '<ul><li>' . implode('</li><li>', $errors) . '</li></ul>';
    return array('error' => array('#value' => $html));
  }*/

  // Check if the newsletter has any content.
  $content_builder = new SunMailerContentBuilder($newsletter);
  $xml = $content_builder->getNewsletterXml();
  if (!$xml) {
    if ($newsletter->hasCutoffDate()) {
      $args = array(
        '!cutoff_url' => url("admin/settings/sunmailer/mail/{$newsletter->getNewsletterId()}/settings"),
        '@cutoff_date' => date('l, F j, Y g:i:s A', $newsletter->getCutoffDate()),
      );
      $html = t('<p>The newsletter has no content. The <a href="!cutoff_url">cutoff date</a> is @cutoff_date.</p>', $args);
    }
    else {
      $html = t('<p>The newsletter has no content.</p>');
    }
    return array('nocontent' => array('#value' => $html));
  }

  // Add preview links.
  $nlid = $newsletter->getNewsletterId();
  $link_options = array('attributes' => array('target' => '_blank'));
  $links = array();
  $links[] = l(t('Preview the newsletter'), "sunmailer/preview/$nlid/design", $link_options);
  if ($newsletter->hasAlternateDesign()) {
    $links[] = l(t('Preview the newsletter (alternate design)'), "sunmailer/preview/$nlid/alternate_design", $link_options);
  }
  $links[] = l(t('Preview the newsletter (XML)'), "sunmailer/preview/$nlid/xml", $link_options);

  $preview = '<ul>';
  foreach ($links as $link) {
    $preview .= "<li>$link</li>";
  }
  $preview .= '</ul>';
  $form['preview'] = array(
    '#type' => 'markup',
    '#value' => $preview,
  );

  // Confirm the mailing.
  $form['confirm'] = array(
    '#type' => 'checkbox',
    '#title' => t('Confirm that you want to send the newsletter'),
  );

  // Submit the form.
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Send'),
  );
  return $form;
}

/**
 * Validation callback for form to send the newsletter.
 */
function sunmailer_mail_send_validate($form, &$form_state) {
  $values = $form_state['values'];
  if (!$values['confirm']) {
    form_set_error('confirm', t('Cannot send without confirmation.'));
  }
}

/**
 * Submission callback for form to send the newsletter.
 */
function sunmailer_mail_send_submit($form, &$form_state) {
  $values = $form_state['values'];
  $newsletter = SunMailerNewsletter::loadNewsletter($values['nlid']);
  sunmailer_send_newsletter($newsletter);
  $form_state['redirect'] = user_access('access site reports') ? 'admin/reports/dblog' : 'admin/settings/sunmailer';
}

/**
 * Generates a form to send a custom message.
 */
function sunmailer_mail_custom(&$form_state = NULL) {
  $newsletters = SunMailerNewsletter::loadAllNewsletters();
  $newsletter_options = array();
  foreach ($newsletters as $nlid => $newsletter) {
    $newsletter_options[$nlid] = $newsletter->getTitle();
  }
  $form = array();

  // Create the custom message.
  if (isset($form_state['storage']['preview'])) {
    $form['#prefix'] = $form_state['storage']['preview'];
  }
  $form['message'] = array(
    '#type' => 'fieldset',
    '#title' => t('Custom message'),
    '#description' => t('Message to email to all subscribers'),
  );
  $form['message']['subject'] = array(
    '#type' => 'textfield',
    '#title' => t('Subject'),
    '#required' => TRUE,
    '#default_value' => isset($form_state['values']) ? $form_state['values']['subject'] : '',
  );
  $form['message']['body'] = array(
    '#type' => 'textarea',
    '#title' => t('Body'),
    '#description' => t('HTML tags will be stripped from the alternate text version'),
    '#required' => TRUE,
    '#default_value' => isset($form_state['values']) ? $form_state['values']['body'] : '',
  );

  $form['nlids'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Recipients'),
    '#description' => t('Leave blank to send to subscribers of all newsletters'),
    '#options' => $newsletter_options,
    '#default_value' => isset($form_state['values']) ? $form_state['values']['nlids'] : array(),
  );

  // Confirm the mailing.
  $form['confirm'] = array(
    '#type' => 'checkbox',
    '#title' => t('Confirm that you want to send this email'),
    '#default_value' => FALSE,
  );

  // Submit the form.
  $form['preview'] = array(
    '#type' => 'submit',
    '#value' => t('Preview'),
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Send'),
  );
  return $form;
}

/**
 * Validation callback for the form to send a custom email.
 */
function sunmailer_mail_custom_validate($form, &$form_state) {
  $values = $form_state['values'];
  if ($form_state['clicked_button']['#value'] != t('Preview')) {
    if (!$values['confirm']) {
      form_set_error('confirm', t('Cannot send without confirmation.'));
    }
  }
}

/**
 * Submission callback for the form to send a custom message.
 */
function sunmailer_mail_custom_submit($form, &$form_state) {
  $values = $form_state['values'];

  // Build the content.
  $subject = $values['subject'];
  $body = filter_xss_admin($values['body']);
  $alt_body = strip_tags($body);

  if ($form_state['clicked_button']['#value'] == t('Send')) {
    // Build the subscriber list.
    $nlids = array_filter($form_state['values']['nlids']);
    $subscription_aggregator = new SunMailerSubscriptionAggregator();
    $addresses = $subscription_aggregator->getAllSubscribers($nlids);

    // Send the message.
    watchdog('mail', 'Sending a custom email message');
    $mailer = new SunMailerMailer();
    $mailer->sendMail($addresses, $subject, $body, TRUE, $alt_body);
    watchdog('mail', 'Sent a custom email message');

    // Redirect.
    unset($form_state['storage']);
    $form_state['redirect'] = user_access('access site reports') ? 'admin/reports/dblog' : 'admin/settings/sunmailer';
  }
  else {
     // Store a preview of the email newsletter.
     $preview  = "<h3>$subject</h3><ul>";
     $preview .= "<li><strong>HTML:</strong> $body</li>";
     $preview .= "<li><strong>Text:</strong> $alt_body</li>";
     $preview .= "</ul>";
     $form_state['storage']['preview'] = $preview;
  }
}
